@@ -92,14 +92,14 @@ namespace types
   template <>
   struct extended_slice<0> {
     template <class T, class pS, class... S>
-    auto operator()(ndarray<T, pS> &&a, long s0, S const &... s)
+    auto operator()(ndarray<T, pS> &&a, long const &s0, S const &... s)
         -> decltype(std::declval<numpy_iexpr<ndarray<T, pS>>>()(s...))
     {
       return std::move(a)[s0](s...);
     }
 
     template <class T, class pS, class... S>
-    auto operator()(ndarray<T, pS> const &a, long s0, S const &... s)
+    auto operator()(ndarray<T, pS> const &a, long const &s0, S const &... s)
         -> decltype(a[s0](s...))
     {
       return a[s0](s...);
@@ -153,6 +153,15 @@ namespace types
     {
       return make_gexpr(a, s0, s...);
     }
+
+    template <class T, class pS, class F, class... S>
+    numpy_gexpr<ndarray<T, array<long, std::tuple_size<pS>::value>>,
+                contiguous_normalized_slice, normalize_t<S>...>
+    operator()(ndarray<T, pS> const &a, F const &s0, S const &... s)
+    {
+      return numpy_vexpr<ndarray<T, pS>, F>{a, s0}(
+          contiguous_slice(none_type{}, none_type{}), s...);
+    }
   };
 
   /* manually unrolled copy function
@@ -304,17 +313,14 @@ namespace types
       auto operator()(long const *s, std::tuple<S0, T0...> const &t0,
                       std::tuple<none_type, T1...> const &t1)
           -> decltype(tuple_push_head(
-              std::get<0>(t0),
-              tuple_push_head(
-                  std::get<0>(t1),
-                  merge_gexpr<std::tuple<T0...>, std::tuple<T1...>>{}(
-                      s + 1, tuple_tail(t0), tuple_tail(t1)))))
+              std::get<0>(t1),
+              merge_gexpr<std::tuple<S0, T0...>, std::tuple<T1...>>{}(
+                  s + 1, t0, tuple_tail(t1))))
       {
         return tuple_push_head(
-            std::get<0>(t0),
-            tuple_push_head(std::get<0>(t1),
-                            merge_gexpr<std::tuple<T0...>, std::tuple<T1...>>{}(
-                                s + 1, tuple_tail(t0), tuple_tail(t1))));
+            std::get<0>(t1),
+            merge_gexpr<std::tuple<S0, T0...>, std::tuple<T1...>>{}(
+                s + 1, t0, tuple_tail(t1)));
       }
     };
 
@@ -481,8 +487,8 @@ namespace types
       : std::integral_constant<bool, !is_dtype<E>::value> {
   };
 
-  template <class E, size_t N>
-  struct may_overlap_gexpr<array<E, N>> : may_overlap_gexpr<E> {
+  template <class E, size_t N, class V>
+  struct may_overlap_gexpr<array_base<E, N, V>> : may_overlap_gexpr<E> {
   };
 
   template <class Op, class... Args>
@@ -542,6 +548,8 @@ namespace types
                        std::is_same<S, contiguous_normalized_slice>::value ||
                        std::is_same<S, normalized_slice>::value)...>::value,
         "all slices are valid");
+    static_assert(std::decay<Arg>::type::value >= sizeof...(S),
+                  "slicing respects array shape");
 
     // numpy_gexpr is a wrapper for extended sliced array around a numpy
     // expression.
@@ -658,8 +666,7 @@ namespace types
     template <class _Arg, class... _S>
     friend struct details::make_gexpr;
 
-    template <class T, size_t N>
-    friend struct array;
+    friend struct array_base_slicer;
     template <class _Arg, class... _S>
     friend typename std::enable_if<count_new_axis<_S...>::value == 0,
                                    numpy_gexpr<_Arg, _S...>>::type
@@ -1018,18 +1025,47 @@ struct __combined<pythonic::types::numpy_gexpr<Arg, S...>,
 template <class Arg, class... S, class Argp, class... Sp>
 struct __combined<pythonic::types::numpy_gexpr<Arg, S...>,
                   pythonic::types::numpy_gexpr<Argp, Sp...>> {
-  using type = pythonic::types::numpy_gexpr<Arg, S...>;
+  using t0 = pythonic::types::numpy_gexpr<Arg, S...>;
+  using t1 = pythonic::types::numpy_gexpr<Argp, Sp...>;
+  using type =
+      pythonic::types::ndarray <
+      typename __combined<typename t0::dtype, typename t1::dtype>::type,
+        pythonic::types::array < long,
+        t0::value < t1::value ? t1::value : t0::value >>
+      ;
 };
 
 template <class Arg, class... S, class O>
 struct __combined<pythonic::types::numpy_gexpr<Arg, S...>, O> {
   using type = pythonic::types::numpy_gexpr<Arg, S...>;
 };
+template <class Arg, class... S, class T>
+struct __combined<pythonic::types::list<T>,
+                  pythonic::types::numpy_gexpr<Arg, S...>> {
+  using type = pythonic::types::list<typename __combined<
+      typename pythonic::types::numpy_gexpr<Arg, S...>::value_type, T>::type>;
+};
+template <class Arg, class... S, class T>
+struct __combined<pythonic::types::numpy_gexpr<Arg, S...>,
+                  pythonic::types::list<T>> {
+  using type = pythonic::types::list<typename __combined<
+      typename pythonic::types::numpy_gexpr<Arg, S...>::value_type, T>::type>;
+};
+template <class Arg, class... S>
+struct __combined<pythonic::types::numpy_gexpr<Arg, S...>,
+                  pythonic::types::none_type> {
+  using type = pythonic::types::none<pythonic::types::numpy_gexpr<Arg, S...>>;
+};
 
 template <class Arg, class... S, class O>
 struct __combined<O, pythonic::types::numpy_gexpr<Arg, S...>> {
   using type = pythonic::types::numpy_gexpr<Arg, S...>;
 };
+template <class Arg, class... S>
+struct __combined<pythonic::types::none_type,
+                  pythonic::types::numpy_gexpr<Arg, S...>> {
+  using type = pythonic::types::none<pythonic::types::numpy_gexpr<Arg, S...>>;
+};
 
 /* combined are sorted such that the assigned type comes first */
 template <class Arg, class... S, class T, class pS>
